﻿using DocumentFormat.OpenXml.Lite.Core.Interfaces;
using DocumentFormat.OpenXml.Lite.Core.Office;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DocumentFormat.OpenXml.Lite.Shared
{
    public class Embedding : ISerializable
    {
        public byte[] Data { get; set; }

        public string FileName { get; set; }

        public Embedding(byte[] data, string filename) 
        {
            this.Data = data;
            this.FileName = filename;
        }

        public byte[] Serialize()
        {
            return Data;
        }

        public void Deserialize(System.IO.Stream content)
        {
            long originalPosition = 0;

            if (content.CanSeek)
            {
                originalPosition = content.Position;
                content.Position = 0;
            }

            try
            {
                byte[] readBuffer = new byte[4096];

                int totalBytesRead = 0;
                int bytesRead;

                while ((bytesRead = content.Read(readBuffer, totalBytesRead, readBuffer.Length - totalBytesRead)) > 0)
                {
                    totalBytesRead += bytesRead;

                    if (totalBytesRead == readBuffer.Length)
                    {
                        int nextByte = content.ReadByte();
                        if (nextByte != -1)
                        {
                            byte[] temp = new byte[readBuffer.Length * 2];
                            Buffer.BlockCopy(readBuffer, 0, temp, 0, readBuffer.Length);
                            Buffer.SetByte(temp, totalBytesRead, (byte)nextByte);
                            readBuffer = temp;
                            totalBytesRead++;
                        }
                    }
                }

                byte[] buffer = readBuffer;
                if (readBuffer.Length != totalBytesRead)
                {
                    buffer = new byte[totalBytesRead];
                    Buffer.BlockCopy(readBuffer, 0, buffer, 0, totalBytesRead);
                }
                Data = buffer;
            }
            finally
            {
                if (content.CanSeek)
                {
                    content.Position = originalPosition;
                }
            }
        }
    }
}
