﻿using DocumentFormat.OpenXml.Lite.PresentationML;
using DocumentFormat.OpenXml.Lite.PresentationML.Presentation.Base;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using IO = System.IO;

namespace DocumentFormat.OpenXml.Lite.Core.Office
{
    public static class Extension
    {
        public static Presentation CreatePresentation(this DocumentManager manager)
        {
            Guid id = DocumentFS.DocumentFS.CreateSandbox();

            PresentationDocument doc = new PresentationDocument(id, DocumentType.Presentation);
            doc.CreateDefault();
            manager.AddDocument(id, doc);

            return doc.Presentation;
        }

        public static bool OpenPresentationDocument(this DocumentManager manager, string path, out Presentation workbook)
        {
            if (IO.Path.GetExtension(path) != ".pptx")
            {
                throw new OpenXmlException("Unknown OOXML file extension: '" + IO.Path.GetExtension(path) + "'!");
            }

            using (IO.FileStream fs = IO.File.Open(path, IO.FileMode.Open))
            {
                workbook = manager.OpenPresentationDocument(fs);
                return workbook != null;
            }
        }

        public static Presentation OpenPresentationDocument(this DocumentManager manager, IO.Stream stream)
        {
            try
            {
                Guid id = DocumentFS.DocumentFS.CreateSandbox();
                DocumentBase docBase = new PresentationDocument(id, DocumentType.Presentation);

                docBase.CreateDefault();
                manager.AddDocument(id, docBase);

                if (manager.OpenDocument(stream, docBase))
                {
                    return (docBase as PresentationDocument).Presentation;
                }
                return null;
            }
            catch (Exception ex)
            {
                throw new OpenXmlException("Cannot parse presentation document. See inner exception for details.", ex);
            }
        }
    }
}
