﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DocumentFormat.OpenXml.Lite.Core.Office;
using P = DocumentFormat.OpenXml.Lite.PresentationML.Presentation.Base;
using DocumentFormat.OpenXml.Lite.PresentationML.Presentation.PresentationElements;
using DocumentFormat.OpenXml.Lite.PresentationML.Presentation.Types;
using DocumentFormat.OpenXml.Lite.PresentationML.Presentation.SlideElements;
using DocumentFormat.OpenXml.Lite.PresentationML.Drawing;
using DMLC = DocumentFormat.OpenXml.Lite.DrawingML.Color;
using DMLE = DocumentFormat.OpenXml.Lite.DrawingML.Effect;
using DMLF = DocumentFormat.OpenXml.Lite.DrawingML.Font;
using DMLG = DocumentFormat.OpenXml.Lite.DrawingML.Geometry;
using DMLGP = DocumentFormat.OpenXml.Lite.DrawingML.Geometry.Properties;
using DMLTx = DocumentFormat.OpenXml.Lite.DrawingML.Text;
using DMLTf = DocumentFormat.OpenXml.Lite.DrawingML.Transformation;
using DMLT = DocumentFormat.OpenXml.Lite.DrawingML.Types;
using DMLS = DocumentFormat.OpenXml.Lite.DrawingML.Style;
using DMLV = DocumentFormat.OpenXml.Lite.DrawingML.Visual;

namespace Samples.Presentation
{
    class Helloworld
    {
        public static void Create()
        {
            P.Presentation pres = DocumentManager.Instance.CreatePresentation();
            pres.SlideSize = new SlideSize()
            {
                ExtentLength = 9144000,
                ExtentWidth = 6858000,
                Type = ST_SlideSize.Screen_4x3
            };
            pres.NoteSize = new NoteSize()
            {
                ExtentLength = 6858000,
                ExtentWidth = 9144000
            };

            P.SlideMaster master = pres.CreateSlideMaster();
            P.SlideLayout layout = pres.CreateLayout(master);
            P.Slide slide = pres.CreateSlide(layout);
            DMLS.Theme theme = pres.CreateTheme(master);

            #region Create Master

            {
                master.SlideData = new CommonSlideData()
                {
                    Name = "Title Slide",
                    ShapeTree = new ShapeTree()
                    {
                        NonVisualGroupShapeProperties = new NonVisualGroupShapeProperties()
                        {
                            NonVisualDrawingProperties = new DMLGP.NonVisualProperties()
                            {
                                Id = 1,
                                Name = ""
                            },
                            NonVisualGroupShapeDrawingProperties = new DMLGP.NonVisualGroupShapeDrawingProperties(),
                            NonVisualProperties = new NonVisualProperties()
                        },
                        GroupShapeProperties = new DMLGP.GroupShapeProperties()
                        {
                            Transform2d = new DMLTf.GroupTransform2d()
                            {
                                Offset = new DMLG.Offset() { X = 0, Y = 0 },
                                Extents = new DMLG.Extent() { ExtentLength = 0, ExtentWidth = 0 },
                                ChildOffset = new DMLG.Offset() { X = 0, Y = 0 },
                                ChildExtents = new DMLG.Extent() { ExtentLength = 0, ExtentWidth = 0 }
                            }
                        }
                    }
                };

                master.ColorSchemeMap = new ColorSchemeMap()
                {
                    Background1 = DMLT.ST_ColorSchemeIndex.Light1,
                    Background2 = DMLT.ST_ColorSchemeIndex.Light2,
                    Text1 = DMLT.ST_ColorSchemeIndex.Dark1,
                    Text2 = DMLT.ST_ColorSchemeIndex.Dark2,
                    Accent1 = DMLT.ST_ColorSchemeIndex.Accent1,
                    Accent2 = DMLT.ST_ColorSchemeIndex.Accent2,
                    Accent3 = DMLT.ST_ColorSchemeIndex.Accent3,
                    Accent4 = DMLT.ST_ColorSchemeIndex.Accent4,
                    Accent5 = DMLT.ST_ColorSchemeIndex.Accent5,
                    Accent6 = DMLT.ST_ColorSchemeIndex.Accent6,
                    Hyperlink = DMLT.ST_ColorSchemeIndex.Hyperlink,
                    FollowedHyperlink = DMLT.ST_ColorSchemeIndex.FollowedHyperlink
                };
            }

            #endregion

            #region Create Layout

            {
                layout.Type = ST_SlideLayout.Text;
                layout.SlideData = new CommonSlideData()
                {
                    ShapeTree = new ShapeTree()
                    {
                        NonVisualGroupShapeProperties = new NonVisualGroupShapeProperties()
                        {
                            NonVisualDrawingProperties = new DMLGP.NonVisualProperties()
                            {
                                Id = 1,
                                Name = ""
                            },
                            NonVisualGroupShapeDrawingProperties = new DMLGP.NonVisualGroupShapeDrawingProperties(),
                            NonVisualProperties = new NonVisualProperties()
                        },
                        GroupShapeProperties = new DMLGP.GroupShapeProperties()
                        {
                            Transform2d = new DMLTf.GroupTransform2d()
                            {
                                Offset = new DMLG.Offset() { X = 0, Y = 0 },
                                Extents = new DMLG.Extent() { ExtentLength = 0, ExtentWidth = 0 },
                                ChildOffset = new DMLG.Offset() { X = 0, Y = 0 },
                                ChildExtents = new DMLG.Extent() { ExtentLength = 0, ExtentWidth = 0 }
                            }
                        }
                    }
                };
            }

            #endregion

            #region Create Slide

            {
                slide.SlideData = new CommonSlideData()
                {
                    ShapeTree = new ShapeTree()
                    {
                        NonVisualGroupShapeProperties = new NonVisualGroupShapeProperties()
                        {
                            NonVisualDrawingProperties = new DMLGP.NonVisualProperties()
                            {
                                Id = 1,
                                Name = ""
                            },
                            NonVisualGroupShapeDrawingProperties = new DMLGP.NonVisualGroupShapeDrawingProperties(),
                            NonVisualProperties = new NonVisualProperties()
                        },
                        GroupShapeProperties = new DMLGP.GroupShapeProperties()
                        {
                            Transform2d = new DMLTf.GroupTransform2d()
                            {
                                Offset = new DMLG.Offset() { X = 0, Y = 0 },
                                Extents = new DMLG.Extent() { ExtentLength = 0, ExtentWidth = 0 },
                                ChildOffset = new DMLG.Offset() { X = 0, Y = 0 },
                                ChildExtents = new DMLG.Extent() { ExtentLength = 0, ExtentWidth = 0 }
                            }
                        }
                    }
                };

                Shape shape;
                slide.SlideData.ShapeTree.Shapes.Add(shape = new Shape()
                {
                    NonVisualProperties = new NonVisualShapeProperties()
                    {
                        NonVisualDrawingProperties = new DMLGP.NonVisualProperties()
                        {
                            Id = 2,
                            Name = "Title1"
                        },
                        NonVisualShapeDrawingProperties = new DMLGP.NonVisualShapeDrawingProperties()
                        {
                            ShapeLocks = new DMLG.Locks.ShapeLocks() { DisallowGrouping = true }
                        },
                        NonVisualProperties = new NonVisualProperties()
                        {
                            Placeholder = new PlaceholderShape() { Type = ST_PlaceholderType.CenteredTitle }
                        }
                    },
                    Properties = new DMLGP.ShapeProperties(),
                    Body = new DMLTx.Body()
                    {
                        Properties = new DMLTx.Properties.BodyProperties(),
                        Styles = new DMLTx.Properties.TextListStyles()
                    }
                });

                DMLTx.Paragraph p;
                shape.Body.Paragraphs.Add(p = new DMLTx.Paragraph());
                p.Elements.Add(new DMLTx.Run()
                {
                    Text = new DocumentFormat.OpenXml.Lite.Shared.SimpleText() { Value = "Hello World!" }
                });
            }

            #endregion

            #region Create Theme

            {
                theme.Elements = new DMLS.ThemeElements()
                {
                    ColorScheme = new DMLC.ColorScheme()
                    {
                        Name = "Sample",
                        Dark1 = new DMLC.Color { ColorElement = new DMLC.SystemColorModel { ColorValue = DMLT.ST_SystemColorVal.WindowTextSystemColor, LastColor = "000000" } },
                        Light1 = new DMLC.Color { ColorElement = new DMLC.SystemColorModel { ColorValue = DMLT.ST_SystemColorVal.WindowSystemColor, LastColor = "FFFFFF" } },
                        Dark2 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "1F497D" } },
                        Light2 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "EEECE1" } },
                        Accent1 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "4F81BD" } },
                        Accent2 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "C0504D" } },
                        Accent3 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "9BBB59" } },
                        Accent4 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "8064A2" } },
                        Accent5 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "4BACC6" } },
                        Accent6 = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "F79646" } },
                        Hyperlink = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "0000FF" } },
                        FollowedHyperlink = new DMLC.Color { ColorElement = new DMLC.RbgColorHexVariant() { Value = "800080" } }
                    },
                    FontScheme = new DMLF.FontScheme()
                    {
                        Name = "Sample",
                        MajorFont = new DMLF.FontElement()
                        {
                            Latin = new DMLF.Font() { Typeface = "Calibri" },
                            EastAsian = new DMLF.Font() { Typeface = "" },
                            ComplexScript = new DMLF.Font() { Typeface = "" }
                        },
                        MinorFont = new DMLF.FontElement()
                        {
                            Latin = new DMLF.Font() { Typeface = "Calibri" },
                            EastAsian = new DMLF.Font() { Typeface = "" },
                            ComplexScript = new DMLF.Font() { Typeface = "" }
                        }
                    },
                    FormatScheme = new DMLS.FormatScheme()
                    {
                        Name = "Sample"
                    }
                };

                theme.Elements.FormatScheme.LineStyles.Add(new DMLG.Line());
                theme.Elements.FormatScheme.LineStyles.Add(new DMLG.Line());
                theme.Elements.FormatScheme.LineStyles.Add(new DMLG.Line());

                theme.Elements.FormatScheme.FillStyles.Add(new DMLV.NoFill());
                theme.Elements.FormatScheme.FillStyles.Add(new DMLV.NoFill());
                theme.Elements.FormatScheme.FillStyles.Add(new DMLV.NoFill());

                theme.Elements.FormatScheme.BackgroundFillStyles.Add(new DMLV.NoFill());
                theme.Elements.FormatScheme.BackgroundFillStyles.Add(new DMLV.NoFill());
                theme.Elements.FormatScheme.BackgroundFillStyles.Add(new DMLV.NoFill());

                theme.Elements.FormatScheme.EffectStyles.Add(new DMLE.EffectStyle()
                {
                    EffectProperty = new DMLE.EffectDag()
                });
                theme.Elements.FormatScheme.EffectStyles.Add(new DMLE.EffectStyle()
                {
                    EffectProperty = new DMLE.EffectDag()
                });
                theme.Elements.FormatScheme.EffectStyles.Add(new DMLE.EffectStyle()
                {
                    EffectProperty = new DMLE.EffectDag()
                });
            }

            #endregion

            pres.Save(@"samples\helloworld.pptx");

            P.Presentation parsed = null;
            if (DocumentManager.Instance.OpenPresentationDocument(@"samples\helloworld.pptx", out parsed))
            {
                DMLS.Theme parsedTheme = parsed.FindThemes()[0];
                P.SlideLayout parsedLayout = parsed.FindLayoutForSlide(parsed.Slides[0]);
                Console.WriteLine("Sample created and parsed succesfully!");
            }
            Console.ReadLine();
        }
    }
}
