﻿using DocumentFormat.OpenXml.Lite.Core.ContentType;
using DocumentFormat.OpenXml.Lite.Core.Interfaces;
using DocumentFormat.OpenXml.Lite.Core.Office;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DocumentFormat.OpenXml.Lite.Shared
{
    public class Image : ISerializable, IDrawingElement
    {
        private byte[] data;

        public ImageType Type { get; set; }

        public Image(byte[] data, ImageType type) 
        {
            this.data = data;
            this.Type = type;
        }

        public string ContentType
        {
            get
            {
                switch (Type)
                {
                    case ImageType.BMP:
                        return "bmp";
                    case ImageType.GIF:
                        return "gif";
                    case ImageType.JPG:
                        return "jpg";
                    case ImageType.JPEG:
                        return "jpeg";
                    case ImageType.PNG:
                        return "png";
                    case ImageType.TIFF:
                        return "tiff";
                    default: break;
                }
                return string.Empty;
            }
        }

        public string MimeType
        {
            get
            {
                switch (Type)
                {
                    case ImageType.BMP:
                        return "image/bmp";
                    case ImageType.GIF:
                        return "image/gif";
                    case ImageType.JPG:
                        return "image/jpg";
                    case ImageType.JPEG:
                        return "image/jpeg";
                    case ImageType.PNG:
                        return "image/png";
                    case ImageType.TIFF:
                        return "image/tiff";
                    default: break;
                }
                return string.Empty;
            }
        }

        public byte[] Serialize()
        {
            return data;
        }

        public void Deserialize(System.IO.Stream content)
        {
            long originalPosition = 0;

            if (content.CanSeek)
            {
                originalPosition = content.Position;
                content.Position = 0;
            }

            try
            {
                byte[] readBuffer = new byte[4096];

                int totalBytesRead = 0;
                int bytesRead;

                while ((bytesRead = content.Read(readBuffer, totalBytesRead, readBuffer.Length - totalBytesRead)) > 0)
                {
                    totalBytesRead += bytesRead;

                    if (totalBytesRead == readBuffer.Length)
                    {
                        int nextByte = content.ReadByte();
                        if (nextByte != -1)
                        {
                            byte[] temp = new byte[readBuffer.Length * 2];
                            Buffer.BlockCopy(readBuffer, 0, temp, 0, readBuffer.Length);
                            Buffer.SetByte(temp, totalBytesRead, (byte)nextByte);
                            readBuffer = temp;
                            totalBytesRead++;
                        }
                    }
                }

                byte[] buffer = readBuffer;
                if (readBuffer.Length != totalBytesRead)
                {
                    buffer = new byte[totalBytesRead];
                    Buffer.BlockCopy(readBuffer, 0, buffer, 0, totalBytesRead);
                }
                data = buffer;
            }
            finally
            {
                if (content.CanSeek)
                {
                    content.Position = originalPosition;
                }
            }
        }
    }
}
