﻿using DocumentFormat.OpenXml.Lite.Core.Office;
using DocumentFormat.OpenXml.Lite.Shared;
using DocumentFormat.OpenXml.Lite.Helper;
using DocumentFormat.OpenXml.Lite.Xml;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using IO = System.IO;
using System.Xml.Serialization;
using DocumentFormat.OpenXml.Lite.WordprocessingML.Word.SectionProperties;
using System.Xml;
using DocumentFormat.OpenXml.Lite.Core.Interfaces;

namespace DocumentFormat.OpenXml.Lite.WordprocessingML.Word.Base
{
    [XmlRoot("hdr", Namespace = Constants.WordMain)]
    public class Header : XmlSerializable, ISerializable
    {
        [XmlIgnore]
        public Guid DocumentId { get; set; }

        [Optional]
        [XmlElement]
        public XmlList<IOfficeElement> Elements { get; set; }

        [XmlIgnore]
        public string RelationshipId { get; set; }

        [XmlNamespaceDeclarations]
        public XmlSerializerNamespaces Namespaces { get; set; }

        public Header()
        {
            Namespaces = new XmlSerializerNamespaces();
            Namespaces.Add("w", Constants.WordMain);
            //ns.Add("xml", Constants.Xml);

            Elements = new XmlList<IOfficeElement>();
            Elements.Changing += Elements_Changing;
        }

        #region Methods

        public Paragraph AddParagraph()
        {
            Paragraph p = new Paragraph();
            p.DocumentId = this.DocumentId;
            Elements.Add(p);
            return p;
        }

        public Table.Table AddTable()
        {
            Table.Table t = new Table.Table();
            t.DocumentId = this.DocumentId;
            Elements.Add(t);
            return t;
        }

        public Table.Table AddTable(int columns, int rows)
        {
            Table.Table t = new Table.Table(columns, rows);
            t.DocumentId = this.DocumentId;
            Elements.Add(t);
            return t;
        }

        #endregion

        #region Events

        void Elements_Changing(object sender, ListEventArgs<IOfficeElement> args)
        {
            if (args.Type == ListEventType.Remove)
                return;

            foreach (var p in args.Objects)
            {
                p.DocumentId = this.DocumentId;
            }
        }

        #endregion
        
        #region ISerializable members

        public byte[] Serialize()
        {
            XmlSerializer serializer = new XmlSerializer(this.GetType());
            using (var stream = new IO.MemoryStream())
            {
                using (var writer = new IO.StreamWriter(stream, Encoding.UTF8))
                {
                    using (var xwriter = XmlWriter.Create(writer))
                    {
                        xwriter.WriteStartElement("w", "hdr", Constants.WordMain);
                        foreach (var ns in Namespaces.ToArray())
                        {
                            if (ns.Name != "w")
                            {
                                xwriter.WriteAttributeString("xmlns", ns.Name, null, ns.Namespace);
                            }
                        }
                        this.WriteXml(xwriter);
                    }
                    return stream.ToArray();
                }
            }
        }

        public void Deserialize(System.IO.Stream content)
        {
            using (var reader = XmlReader.Create(content))
            {
                reader.ReadToDescendant("hdr", Constants.WordMain);
                for (int i = 0; i < reader.AttributeCount; ++i)
                {
                    string s = reader.GetAttribute(i);
                    if (Constants.GetDefaultPrefix(s) != string.Empty)
                    {
                        Namespaces.Add(Constants.GetDefaultPrefix(s), s);
                    }
                }
                this.ReadXml(reader);
            }
        }

        #endregion
    }
}
