﻿using DocumentFormat.OpenXml.Lite.Helper;
using DocumentFormat.OpenXml.Lite.WordprocessingML.Word.Types;
using DocumentFormat.OpenXml.Lite.Xml;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace DocumentFormat.OpenXml.Lite.WordprocessingML.Word.RunProperties
{
    [XmlRoot("color", Namespace = Constants.WordMain)]
    public class Color : XmlSerializable, IRunPropElement
    {
        /// <summary>
        /// Hexadecimal number. Length: 6
        /// </summary>
        [XmlAttribute("val", Namespace = Constants.WordMain, Form = XmlSchemaForm.Qualified)]
        public string Value
        {
            get { return color; }
            set
            {
                if (value == null || (value.Length != 6 && value != "auto") ||
                    (value.Length == 6 && !Helpers.ParseHexadecimalValue(value)))
                {
                    throw new ArgumentException("String: " + value +
                        "is not a valid hexadecimal string or length is not 6!");
                }

                color = value;
            }
        }

        [Optional]
        [XmlAttribute("themeColor", Namespace = Constants.WordMain, Form = XmlSchemaForm.Qualified)]
        public Nullable<ST_ThemeColor> ThemeColor { get; set; }

        /// <summary>
        /// Hexadecimal number. Length: 2
        /// </summary>
        [Optional]
        [XmlAttribute("themeTint", Namespace = Constants.WordMain, Form = XmlSchemaForm.Qualified)]
        public string ThemeColorTint
        {
            get { return themeColorTint; }
            set
            {
                if (value == null || value.Length != 2 || !Helpers.ParseHexadecimalValue(value))
                {
                    throw new ArgumentException("String: " + value +
                        "is not a valid hexadecimal string or length is not 2!");
                }

                themeColorTint = value;
            }
        }

        /// <summary>
        /// Hexadecimal number. Length: 2
        /// </summary>
        [Optional]
        [XmlAttribute("themeShade", Namespace = Constants.WordMain, Form = XmlSchemaForm.Qualified)]
        public string ThemeColorShade
        {
            get { return themeColorShade; }
            set
            {
                if (value == null || value.Length != 2 || !Helpers.ParseHexadecimalValue(value))
                {
                    throw new ArgumentException("String: " + value +
                        "is not a valid hexadecimal string or length is not 2!");
                }

                themeColorShade = value;
            }
        }

        [XmlIgnore]
        private string color;

        [XmlIgnore]
        private string themeColorTint;

        [XmlIgnore]
        private string themeColorShade;

        public Color() { }

        public void Copy(object obj)
        {
            if (obj is Color)
            {
                this.Value = (obj as Color).Value;
                this.ThemeColor = (obj as Color).ThemeColor;
                this.ThemeColorTint = (obj as Color).ThemeColorTint;
                this.ThemeColorShade = (obj as Color).ThemeColorShade;
            }
        }
    }
}
